@@ -109,13 +109,6 @@ class HgImplementation(M.RepositoryImplementation):
 
     def commit(self, rev):
         '''Return a Commit object.  rev can be _id or a branch/tag name'''
-        # See if the rev is a named ref that we have cached, and use the sha1
-        # from the cache. This ensures that we don't return a sha1 that we
-        # don't have indexed into mongo yet.
-        for ref in self._repo.heads + self._repo.branches + self._repo.repo_tags:
-            if ref.name == rev:
-                rev = ref.object_id
-                break
         result = M.repo.Commit.query.get(_id=rev)
         if result is None:
             try:
@@ -167,22 +160,6 @@ class HgImplementation(M.RepositoryImplementation):
             to_visit += parents
         return list(topological_sort(graph))
 
-    def refresh_heads(self):
-        self._repo.heads = [
-            Object(name=None, object_id=self._hg[head].hex())
-            for head in self._hg.heads()]
-
-        self._repo.branches = []
-        for name, tag in self._hg.branchtags().iteritems():
-            if ("close" not in self._hg.changelog.read(tag)[5]):
-                self._repo.branches.append(
-                    Object(name=name, object_id=self._hg[tag].hex()))
-
-        self._repo.repo_tags = [
-            Object(name=name, object_id=self._hg[tag].hex())
-            for name, tag in self._hg.tags().iteritems()]
-        session(self._repo).flush()
-
     def refresh_commit_info(self, oid, seen, lazy=True):
         from allura.model.repo import CommitDoc
         ci_doc = CommitDoc.m.get(_id=oid)
@@ -289,12 +266,11 @@ class HgImplementation(M.RepositoryImplementation):
         return root
 
     def symbolics_for_commit(self, commit):
-        branch_heads, tags = super(self.__class__, self).symbolics_for_commit(commit)
         try:
             ctx = self._hg[commit._id]
         except RepoLookupError as e:
-            return [], tags
-        return [ctx.branch()], tags
+            return [], []
+        return [ctx.branch()], ctx.tags()
 
     def compute_tree_new(self, commit, tree_path='/'):
         ctx = self._hg[commit._id]
@@ -353,4 +329,16 @@ class HgImplementation(M.RepositoryImplementation):
     def is_empty(self):
         return not self._hg or self._hg.heads() == ['\x00'*20]
 
+    @LazyProperty
+    def heads(self):
+        return [Object(name=None, object_id=self._hg[v].hex()) for v in self._hg.heads()]
+
+    @LazyProperty
+    def branches(self):
+        return [Object(name=k, object_id=self._hg[v].hex()) for k,v in self._hg.branchtags().items()]
+
+    @LazyProperty
+    def tags(self):
+        return [Object(name=k, object_id=self._hg[v].hex()) for k,v in self._hg.tags().items()]
+
 Mapper.compile_all()
